Crypto
======

Notation
--------
 * `AE_k(plaintext)` denotes the _authenticated_ encryption of `plaintext` under the key `k`.
 * `RAND_k(plaintext)` denotes the randomized authenticated encryption of `plaintext` under the key `k`.
    + Sometimes we omit `k` when it's obvious or does not matter for clarity/succinctness 
    + No authentication of plaintext/ciphertext is provided here, so ciphertext can be malleable
 * `DET_k(plaintext)` denotes the deterministic encryption of `plaintext` under the specified key given by `k`.
 * `SIG_k(m)` denotes a signature on the message `m` with the key given by `k`
 * `MAC_k(m)` denotes a message authentication tag on the message `m` with the key given by `k`

Type of objects
---------------
 - Photos
 - Albums
 - Comment
 - Likes
 - Comment threads
 - Private message or public post
 - Private conversation or public wall

Objects have object IDs (`objid`).

**Simple objects (no versioning, never change, single owner or author):** 

 - One _photo_
   + `obj = ("photo", owner, date, aclVer, RAND_photoK(photo_attrs, photo_bits)), objid = H(obj)`
     * A photo's object ID is the hash of its encryption
     * Where, `aclVer` is the version of the ACL associated with this photo. It tells us which version
       of the ACL to lookup and find the photo key in
       + Initially, `aclVer = 1` when an album is created. As users are removed/added from the album, the 
         `aclVer` is incremented and new photos added to the album use that `aclVer`.
     * Where, `date` is the date when the photo was uploaded or posted (the date when the photo was taken
       can be stored in `photo_attrs`).
 - One wall post / message / comment (a.k.a. _quacks_)
   + **Approach B:** `obj = ("quack", type, date, aclVer, prevQuackId, author, RAND_dstGroupK(msg)), objid = H(obj), sig = SIG_srcUserSK(obj)`
     * Where, `quackType = wall post | message | comment"` indicates the type of the _quack_
     * Note that the signature includes the previous _quack_, which in turn, has a signature on the _quack_ before it, and so on...
     * Note that we cannot keep the `author` secret because his signature gives him away
     * Where `prevQuackId` is:
       - the object ID of the parent object that the _dialogue_ is associated with (if this is the head _quack_ node in the _dialogue_), which can be
         + the object ID of a photo or an album
         + the object ID of a _wall post_
       - the object ID of a _dialogue_
         + the object ID of a conversation
         + the object ID of a wall
       - This is needed to prevent users colluding with the server from copying or moving _quacks_ from
         one wall / conversation / comment-thread into another wall / conversation / comment thread. If we 
         don't include the previous _quack_ ID, then the user is only attesting of having said "something" 
         but he is not attesting of having said "something within a specific conversation." Context is important :P
     * Where `prevQuackHash` is:
       - the object ID of the previous _quack_ (for non-head _quack_ nodes)
     * Now people must verify all new wall posts / messages / comments are "tied-in" to the last seen messages via the signature chaining
       - `=> O(# of new quacks)` verify operations, but we would've done these anyway before as we checked all the signatures on each new _quack_
     * `dstGroupK` is the key of the destination user or group, whose version is specified by `aclVer`
       - Can be a _wall key_ or a _conversation key_ or a _comment thread key_
         + Comment thread key can be the same as the key for the content that was commented on
           * We want to avoid rekeying too many comment threads.
     * `srcUserSK` is the private key of the user who sent the _quack_
     * the signature is needed for _non-repudiability_ of the _quack_ and for integrity of the chain
 - One _like_
   + `SIG_srcUserSK("like", objid)`
     * Don't want server to lie about someone "liking" something
     * perhaps "likes" need to be verified slowly over time, we don't want to have to verify hundreds of "likes"
       every time some photo is displayed
     * signature for _non-repudiability_

Merkle Tree versus a signature chain for _dialogues_: 

 - Merkle Trees are good for verifying arbitrary _quacks_, but they require a signature on the root which forces us to use a MAC for shared content 
   where other users can modify it like conversations. Using a MAC means any _dialogue_ member can certify the state of the _dialogue_, and so
   any _dialogue_ member can arbitrarily alter the view of the _dialogue_. They cannot alter the _quacks_ themselves, because they are signed by the author,
   but they can exclude _quacks_, or duplicate _quacks_ maybe (within the same _dialogue_).
 - Given an authentic root hash, to verify a message, `O(log n)` work has to be done for each message. So up to `O(m log n)` work for `m` new messages.
 - With signature chains, we need `O(m)` work to verify `m` new messages, assuming we have the most recent signature before the new messages came in.
 - None of these schemes fare well when there is a large number `m` of new updates.
   + With Merkle Trees, we would just trust the first `m-k` ones and verify the `k` displayed ones, but we could be forked by malicious _dialogue_ participants or the server.
   + With signature chains, we would have to verify all the `m` signatures link back to the top of the chain. Otherwise, a whole new chain could be started. Is this worse than Merkle Trees? A whole new Merkle Tree can be started as well.
 - Since we always look at new messages and assume past messages are okay, it seems that Merkle Trees don't give us much (other than implementation effort :P).

**Composite objects (versioned, subject to ACLs):**

 - _Albums_
   + `objid = random(), obj = ("album", owner, date, aclVer, objid, RAND_albumK(album_name, album_attrs), rootHash), sig = SIG_ownerSK(obj), rootHash = merkleTree(objid1, objid2, ..., objidn)`
     * Where, `objid1, objid2, ..., objidn` are the object IDs of the photos in the album
     * Where, `aclVer` is incremented every time a user leaves an album 
     * Removing photos from an album involves replacing its object ID with a deleted photo object ID
       - `delObjId = all zeros` or `delObjId = H("photo", ..., deleted, ...)`
       - ...and updating the Merkle tree
       - Can fill these empty spots up later.
     * Adding objects to an album can be done by appending a new leaf and "hooking it up" to the Merkle tree by creating the necessary nodes with the necessary empty subtrees
 - Walls or conversations or comment threads (a.k.a. _dialogues_)
   + **Approach B:** `dialogue = ("dialogue", type, owner, title, lastQuackId)`
     * Where, `dialogueType = wall | conversation"` indicates the type of the _dialogue_
       + Comment threads no longer have a _dialogue_ object associated with them. They just attach directly to the content using their `prevQuackObjId` field
     * The client can cache the `lastSignature` and verify new updates against it
     * Where, `objid1, objid2, ..., objidn` are the object IDs of the messages/posts/comments in the conversation/wall/comment-thread
     * Where, `owner` is the:
       - conversation starter, for a private or group message
       - the owner of the wall, for a wall
       - the owner of the content commented on, for a comment thread
     * Where, `aclVer` is incremented every time a user is defriended or leaves a conversation
     * Where, `dialogue_title` is:
       - the tile of the conversation, for a conversation
       - empty for a wall/comment thread

**Linked objects (no fork consistency):**

 - Photos + Likes + Comment thread
 - Albums + Likes + Comment thread
 - Posts/messages + Likes

Access control lists (ACL)
--------------------------

Every shared object, like a wall, an album, a conversation or a comment thread has a key shared by users who have access to it. Users can be kicked out directly (from a conversation for instance) or indirectly from Alice's wall or his album for instance by defriending Alice.

**Approach #1:** Encrypt each shared key for every user with access to that content.
 - Adding a user: just encrypt the key for him: `O(1)`
 - Removing a user: generate new key and reencrypt the key for all n users: `O(n)`
 - Finding the key for a user: direct lookup: `O(1)`

**Approach #2:** Store the shared key in the root node of a tree, encrypted under the left child's key and the right child's key. Apply recursively for child nodes. There will be a leaf for every user `u` that stores its key encrypted under that user `u`'s key (this will be the friend key shared between the user `u` and the owner of the ACL).
 - Adding a user: must add new leaf to tree and encrypt up to the root node: `O(log n)`
 - Removing a user: must change all keys along the path to the root and reencrypt each one twice for its children: `O(log n)`
 - Finding the key for a user: binary search if tree is balanced: `O(log n)` 

The ACL tree needs integrity as well, so we have a Merkle tree on top of it. The root hash is signed along with:
 - the `objid` of the object the ACL is for
 - the `owner` of the object (and of the ACL tree as a result)
 - the `aclVer` (ACL version) corresponding to this tree

**TODO:** 

 * [DONE] In approach B for the ACL version of a dialogue, now the users themselves can trick other users into using an old ACL version, so that's not good.
   They would need the server to just change the version in the DB and the client to forget the last seen ACL version.
   - This is not that different from a typical server-user colluding scenario though
 * [DONE] If we maintain a Merkle Tree over all the albums, then if multiple devices update different albums they can fork each other off.
   - Device one changes: `T -> T1`
   - Device two changes the same `T`: `T -> T2`
   - Now the first device will think it's being attacked or something when it sees a different root hash `T2` instead of `T1` which it saved locally.
   - This can also happen if two devices upload a new picture to the same album `=>` must lock root hash temporarily before changing it
 * [DONE] Figure out how to comment on wall posts
   - link to the wall post using the `prevQuackObjId` field in the comment
 * [DONE] Figure out how to efficiently bind some objects together, like comments with their parent objects. Seems like an update to a comment thread needs to update the Merkle Tree of its parent object.
   - no freshness guarantees for comment threads
 * [DONE] Figure out how comment threads are keyed
   - use key of object commented on
 * [DONE] Figure out how to "unlike" securely so that other users don't learn you've "liked" and "unliked"
   - server has to NOT send the "like" signature immediately
   - "unliking" securely would mean trusting the server to not send your previous "like" signature
 * [DONE] Need integrity on friend keys, must make sure they never change though, to make our job easier
   - have to use merkle trees and strong freshness guarantees for friend lists. we don't want to send a new 
     album key for instance to a defriended friend
 * [DONE] For composite objects that are MAC'd, like comment threads it seems that I can copy a comment from one thread to another and convince the members that someone has posted that message there...
   - One solution is to include the parent object ID in the MAC/signature of the comment. This binds the comment with the comment thread or photo album or wall post or whatever.
   - Another solution is to use hash chains, which binds a comment with the previous comment, ultimately leading to the parent object ID
 * [DONE] Is version field needed?
   - ACL version is needed because we need to bind the change of ACL to the new object.
   - As for the object version, we use the `date` field for that.
 * [DONE] Tie in ACLs
   - Must make sure that given an `aclVer` included in a signed object, a user can verify the downloaded ACL tree is the right tree for **that object** and **that version**
   - ACL edits are only done or processed by `owner`
   - Members must all be able to verify binding of ACL to object, but should not be able to forge one `=>` cannot use MACs
   - **Solution:** Maintain a Merkle Tree over the ACL tree, signed by the `owner` at the top, including the `aclVer` and the `objid` of the content
 * [DONE] Must include members in MAC/SIG of root hash of composite objects
   - This is a consequence of tying in the ACL to the composite object
 * [DONE] How to deal with delay between people leaving shared content and them being removed by the owner from the ACL?
   - We can simply consider that delay as not being dangerous because the user chose to leave himself
   - The dangerous situation is when the user is kicked out

Secrecy model
-------------
All user submitted photos, messages and comments are kept confidential to the people they are shared with. Specifically, if a user `u` is on the ACL version `v` (i.e. `ACL(v)`) for content `c` then his updates to that content `c` are kept secret from:
 - all users `u` s.t. `\exists j <= v, s.t. u \in { ACL(i), i < j } AND u \notin { ACL(i), i >= j }` (secrecy from users who were removed from a version smaller than or equal to `v`)
 - all users `u` s.t. `u \notin { ACL(i), \forall i }` (secrecy from users who were never granted access)

Forking attacks can allow a webserver to trick a user into revealing new content updates. The user can be tricked to send these updates to users with revoked access. However, these attacks are limited in several ways:
 - the tricked user needs to have his browser cookies cleaned 
 - the tricked user needs to make the mistake and **not see** that a revoked user is part of the shared content (like in a conversation)
   + this does not apply to comment threads or walls or photo albums, where the members with access are not displayed because there would be too many
     * unfortunately, the best a tricked user can try to do is ask himself if the presented content is older than expected
 - the tricked user will only reveal the new message / comment / wall post / photo he posted. previously posted content remains secret.
 - the tricked user has to be forked by the server and can not be brought back to the old view of the content, unless his browser cookies are cleaned again
   + the server cannot join the forked view with the old view due to signature chains. Thus, the tricked user can notice that some new content is missing once he is brought back.

Integrity & freshness model
---------------------------
We use signatures on _quacks_, which give us integrity (and _non-repudiability_)
for each _quack_. We also chain them in a signature chain, so we get
fork-consistency for a _dialogue_.

For albums, we use Merkle trees to provide integrity. We sign the root hash of
the Merkle tree to prevent the server from arbitrarily building trees and thus
modifying picture ciphertexts or reordering them or moving them across albums.

Assuming a permanent browser cache (which can be true for mobile apps for
instance, where your app's data is permanent) we provide fork-consistency on an
_individual object_ basis (album, comment thread, conversation, wall), as
defined in "Building secure file systems out of Byzantine storage":
 
 > However, we define a notion of data integrity called fork consistency in which, 
 > if the server delays just one user from seeing even a single change by another, 
 > the two users will never again see one another’s changes--a failure easily 
 > detectable with on-line communication.
 
 > A file system with fork
 > consistency might conceal users’ actions from each other, but if it
 > does, users get divided into groups and the members of one group
 > can no longer see any of another group’s file system operations.
 > Users can detect such partitioning through any available out-of-
 > band communication—from conversation ("Did you see the new
 > file?"), to client-to-client gossip protocols, to trusted on-line version-
 > verification devices.

This is good enough for a few reasons:
 - we can always bind more objects together via Merkle trees and widen the
   consistency to multiple objects
 - once the server forks someone's view of an object, it cannot join them back
   to that previous view 
   * unless their browser cache is cleared, but even then, the server cannot
     merge the updates from the forked view into the previous view

The server (like any other server) can prevent people from downloading new
updates (like new messages, wall posts, likes, etc) and there is not much
we can do regarding that other than enforcing fork-consistency which forces the
server to give us all updates or otherwise clients might be able to
detect differences.

Adding friends
--------------
 * User needs to keep an authenticated list of friends on the server
 * Map from `friend's username` to `RAND(friend key), MAC(friend's public key, identity, friend key)`
   + Never use a key until you verify the MAC
   + The friend key never changes (unless you defriend and refriend again)
 * For out of network people (more than 2 friends aways)
   - public key authentication (with OOB auth of public keys)

Key management and revocation
-----------------------------
 * Generate new key `k'`, fork off a new ACL tree.
 * Server can actively replay old ACL tree to unsuspecting users who will then
   use the old key `k` to encrypt new content
   + It's impossible to let these user know via the server that the old key `k`
     should never be used again
   + But these users will see the list of people who also share key `k` so they
     always know _who_ they are communicating to
     - Propagating updates about who is not trusted anymore can clearly not be
       done via an untrusted server that is willing to drop messages at any point

SJCL
----
Not clear how to init randomness.
46x slower than OpenSSL on fastest Chrome Browser at the time of writing https://bitwiseshiftleft.github.io/sjcl/acsac.pdf
