## Hash Chains

### Table

    HashChain = {
        id: AutoInteger
        headNode: HashChainNode ID
    }

    HashChainNode = {
        id: AutoInteger
        value: Bytes
        hash: Bytes     
        prevNode: Null or HashChainNode ID
    }

### Methods

* insert(hashchain: HashChain ID, value: Bytes)
    * Adds a HashChainNode with value value to the head of the hash chain whose ID is hashchain.
    * Returns headNode.hash, the hash in the new headNode.

* getKValues(hashchain: HashChain ID, number_of_values: integer)
    * Return the top number_of_values number of entries in the hash chain, along with the hash (HashChainNode.value, HashChainNode.hash).

* deleteTop(hashchain: HashChain ID)
    * Sets headNode = headNode.prevNode. Deletes whatever entries are no longer needed?
    * Returns? What will be useful? 
    

## Merkle Trees

### Table

    MerkleTree = {
        id: AutoInteger
        rootNode: MerkleTreeNode ID
        leafNodes: [ MerkleTreeNode IDs ] array
        emptyLeafNodes: [ MerkleTreeNode IDs] array 
        treeDepth: Integer
    }

    MerkleTreeNode = {
        id: AutoInteger
        isEmpty: Boolean
        hash: Bytes    
        parent: Null or MerkleTreeNode ID
        leftChild: Null or MerkleTreeNode ID
        rightChild: Null or MerkleTreeNode ID
    }

    MerkleTreeLeafIndex = {
        treeId: MerkleTree ID
        value: Bytes
        nodeId: MerkleTreeNode ID
    }

### Methods

* insert(merkletree: MerkleTree ID, value: Bytes)
    * Inserts value value into merkletree.
        * If emptyLeafNodes is not empty, take a purported empty node from it, check that this node is indeed empty by checking that node.isEmpty=True and that the node.hash = 0, and by verifying this using the Merkle tree itself. Note that we defer verifying the integrity of the root hash to whichever procedure is operating on this Merkle tree, which checks that the root hash is signed - the signature would be stored in the enveloping table that contains this Merkle tree, for example the Album table. If it checks out as an empty node, we will use it and delete this from emptyLeafNodes.
        * If emptyLeafNodes is empty, we need a new node. We ask the server for this empty node and verify as before. One approach to handle this easily is to always allocate full Merkle trees. Whenever a tree is filled and there are no empty nodes, we double its size, adding one more level to the tree, and adding all newly created nodes to the emptyLeafNodes list. This gives good amortised running time, but not for every operation. I have an idea to get the same complexity, but guaranteed for every operation - it is slightly difficult to describe here, but somewhat straighforward.
        * Once we have an empty node, we fill in its value, set node.isEmpty=False, and update hashes up the tree till the root. Again, the caller is responsible for maintaining signatures on the root.
    * Adds (merkletree, value, node) to MerkleTreeLeafIndex.    
    * Returns the root hash of the resulting tree.    

* delete(merkletree: MerkleTree ID, merkletreenode: MerkleTreeNode ID)
    * Removes the specified node from the merkletree.
    * First checks that merkletreenode is in fact in the tree corresponding to merkletree, by going up the tree till the root and verifying that it is the same as merkletree.rootNode, and that it is indeed a leaf node. We currently do not have a way to securely check that it is a leaf, but it seems like these checks are not really necessary. Does the following argument for this make sense? First, for all the Merkle trees that we use, we require a signature on the root for it to be accepted as a valid tree by any client. So any call to delete whose result will be used later will have to be done by someone who can sign the root, and we trust anyone whose signature we accept on the root. So any call to delete is only made by a trusted party.      
    * Sets merkletreenode.isEmpty=True, merkletreenode.hash=0, and updates all the nodes up to the root, and adds the node to merkletree.emptyLeafNodes.  
    * Deletes (merkletree, *, merkletreenode) from MerkleTreeLeafIndex.
    * Returns the root hash of the resulting tree.    

* verify(merkletree: Merkletree ID, merkletreenode: MerkleTreeNode ID, value: Bytes)
    * Checks that merkletreenode.hash=value, that all the hashes from this node up to the root check out, and that the root is the rootNode of merkletree.
    * Returns True or False.
        

