// TODO: don't allow people to add their friends unless the convo owner is friends with them too
// due to public key exchange fiasco

Template.convoList.helpers({
  conversations: function() {
    check(this.me, String);
    assert(this.myMasterKey);

    var conversations = [];
    var ids = Dialogue.getConversationIDs(this.me);

    for(var i = 0; i < ids.length; i++) {
      var convo = Dialogue.getConversation(ids[i]);
      assert(convo);
      var owner = convo.data.owner;
      check(owner, String);
      var ownerPublicKey = owner == this.me ? this.myPrivateKey : 
        Friends.getFriendPublicKey(this.me, this.myMasterKey, owner);
      var acl = Acls.getAcl(convo.getObjectId(), owner, ownerPublicKey, 0);
      var aclIndexes = Acls.getMembers(acl.acl._id, acl.rootNodeHash);
      var members = _.pluck(aclIndexes, 'username');

      conversations.push({
        id: convo.data._id,
        owner: owner,
        title: convo.data.title,
        date: convo.data.date, 
        members: members,
        acl: acl,
        aclHash: acl.hash,
        aclIndexes: aclIndexes,
      });
    }
    
    return conversations; 
  },
});

Template.startConvo.created = function() {
  Session.set('searched', false);
  Session.set('matchingFriends', []);
  Session.set('searchQuery', '');
  Session.set('added', {});
};

Template.startConvo.helpers({
  results: function() {
    return Session.get('matchingFriends');
  },

  searchQuery: function() {
    return Session.get('searchQuery');
  },

  added: function() {
    var dict = Session.get('added');
    var arr = [];
    for(var k in dict)
      if(dict[k])
        arr.push(k);
    return arr;
  },
  
  searched: function() {
    return Session.get('searched');
  },
});

Template.startConvo.events({
  'submit .convo-friend-search': function(event, template) {
    event.preventDefault();
    
    Session.set('searched', true);

    var me = this.me;
    var username = template.find('#searchbox').value;
    var added = Session.get('added');
    Session.set('searchQuery', username);

    var matchingUsers = Utils.findUsersRegex(me, username);
    var matchingFriends = [];

    for(var i in matchingUsers) {
      var u = matchingUsers[i];
      //console.log('checking ' + u.username);
      if(Friends.isMyFriend(me, u.username) && !added[u.username]) {
        matchingFriends.push(u.username);
      }
    }

    //console.log('matching friends: ' + stri(matchingFriends));
    Session.set('matchingFriends', matchingFriends);
  },

  'click .add-to-convo': function(event, template) {
    event.preventDefault();

    var added = event.target.id;
    added = added.slice(4); // removes add- prefix
    var arr = Session.get('added');

    arr[added] = true;
    Session.set('added', arr);

    // Remove user from search results
    var matchingFriends = Session.get('matchingFriends');
    matchingFriends = matchingFriends.filter(function(v) { return v != added; });
    Session.set('matchingFriends', matchingFriends);
    if(matchingFriends.length == 0) {
      Session.set('searched', false);
    }
  },

  'click .start-convo': function(event, template) {
    event.preventDefault();
    var added = Session.get('added');
    var to = '';
    for(var k in added) {
      to += k + ','
    }

    to = to.slice(0, to.length - 1);

    Router.go('/newconversation?to=' + to);
  },
  
  'click .remove-from-convo': function(event, template) {
    event.preventDefault();

    var removed = event.target.id;
    removed = removed.slice('remove-'.length); // removes remove- prefix

    var arr = Session.get('added');
    arr[removed] = false;
    Session.set('added', arr);
    
    arr = Session.get('matchingFriends');
    arr.push(removed);
    Session.set('matchingFriends', arr);

    //console.log('removed ' + removed);
  }
});
