Template.friends.helpers({
  friends: function() {
    if(!Meteor.user())
      return Utils.notLoggedInErrorMsg;

    return Friends.getFriends(Utils.me());
  },
});

Template.friendRequests.helpers({
  friendRequests: function () {
    // Problem 1: The template gets called right after I log in but before I am fully logged in
    // so I need this call here.
    if(!Meteor.user())
      return Utils.notLoggedInErrorMsg;

    // Problem 2: The template gets called three times. The first time fails the DB query
    // as if the DB row did not exist. The next 2 times it succeeds. But it should only be
    // called once.
    var reqs = Friends.getFriendRequests(Utils.me());

    _.each(reqs, function(element, it, list) {
      check(element.initiator, String);
      // Get the user's fingerprint
      element.fingerprint = Utils.getFingerprintByUsername(element.initiator);
    });
    //console.log('reqs: ' + EJSON.stringify(reqs));
    //console.log('reqs[0].initiator: ' + EJSON.stringify(reqs[0].initiator));

    return reqs;
  },
});

Template.searchForm.created = function () {
  Session.set('searchFound', false);
  Session.set('searchResults', []);
  Session.set('searchError', '');
};

Template.searchForm.events({
  'submit form': function(event, template) {
    event.preventDefault();
    var me = Utils.me();
    var username = template.find('#searchbox').value;
    //console.log('searching for: ' + username);
    //console.log('users: ' + EJSON.stringify(Meteor.users.find().fetch()));
    var found = Utils.findUsersRegex(me, username); 

    //console.log('all matches found: ' + EJSON.stringify(found));

    var dict = {};
    var friends = Friends.getFriends(me);

    // Remove current friends from the search results
    //console.log('friends: ' + friends);
    _.each(friends, function(element, it, list) {
        //console.log('el: ' + element);
        //console.log('list[it]: ' + list[it]);
        dict[list[it]] = true;
    });
    //console.log('dict: ' + EJSON.stringify(dict));
    found = found.filter(function(p) {
      return dict[p.username] != true;
    });
    //console.log('filtered: ' + EJSON.stringify(found));

    // Compute each user's public key fingerprint
    _.each(found, function(element, index, list) {
      //console.log('element: ' + EJSON.stringify(element));
      //console.log('pubkey: ' + element.profile.publicKey);
      element.fingerprint = CryptoUtils.getPublicKeyFingerprint(element.profile.publicKey);
      element.pendingFriendRequest = 
        Friends.isFriendRequestSent(me, element.username) ||
        Friends.isFriendRequestSent(element.username, me);
      console.log(element.username + '.pendingFriendRequest: ' + element.pendingFriendRequest);
    });

    //console.log('found: ' + EJSON.stringify(found));
    Session.set('searchFound', found.length > 0);
    if(found.length == 0) {
      Session.set('searchError', 'ERROR: Could not find username \'' + username + '\'');
    }
    Session.set('searchResults', found);
  }
});

Template.searchResults.helpers({
  results: function() {
    var arr = Session.get('searchResults');
    //console.log('array of results: ' + EJSON.stringify(arr));
    return arr;
  },

  found: function() {
    return Session.get('searchFound') == true;
  },

  error: function() {
    //console.log('searchError: ' + Session.get('searchError'));
    return Session.get('searchError');
  }
});
