Template.login.events({
  'submit form': function (event, template) {
    event.preventDefault();
    var username = template.find('#login-username').value;
    var password = template.find('#login-password').value;

    Meteor.loginWithPassword(username, password, function(hasError) {
      if(hasError) {
        //console.log('login error: ' + EJSON.stringify(hasError, { indent: true }));
        //console.log(template.find('#error'));
        template.find('#error').textContent = 'ERROR: ' + hasError.reason;
      } else {
        //console.log('Meteor.user().profile: ' + EJSON.stringify(Meteor.user().profile));
        var masterKey = Utils.deriveMasterKey(password, Meteor.user().profile.masterKeySalt);
        var encPrivKey = Meteor.user().profile.encAuthKeyPair.priv;
        var keyPair = CryptoUtils.unserializePublicKey(
          CryptoUtils.decrypt(masterKey, encPrivKey)
        );

        Trustore.Keys.set('master', masterKey);
        Trustore.Keys.setAsym(keyPair);

        console.log('great login success!');

        Router.go('/');
      }
    });

    console.log('login form submitted!');
  },
  'click a.loginRandom': function(event, template) {
    registerRandomUser(template);
  }
});

Template.register.events({
  'submit form': function (event, template) {
    event.preventDefault();
    var username = template.find('#register-username').value;
    var email = template.find('#register-email').value;
    var password = template.find('#register-password').value;

    registerUser(username, email, password, template);

    console.log('registration form submitted!')
  },
  'click a.registerRandom': function(event, template) {
    registerRandomUser(template);
  }
});

function registerRandomUser(template) {
  var username = 'randuser' + CryptoUtils.randomInt(0, 100);
  registerUser(username, username + '@dot.com', username, template);
  console.log('registered user \'' + username + '\'');
}

function registerUser(username, email, password, template) {
  Utils.registerAccount(username, email, password, function(hasError, creds) {
    if(hasError) {
      //console.log('registration error: ' + EJSON.stringify(hasError, { indent: true }));
      template.find('#error').textContent = 'ERROR: ' + hasError.reason;
    } else {
      Trustore.Keys.set('master', creds.masterKey);
      Trustore.Keys.setAsym(creds.asymKeyPair);
      Trustore.Keys.set('profile', creds.profile.profileKey);
      // NOTE: This is already set in createAcl
      //Trustore.Hashes.set('acl.' + creds.profile.profileId, creds.profile.profileAclTree.hash);
      console.log('great registration success!');
      Router.go('/');
    }
  });
}
