Template.profile.helpers({
    myProfile: function() {
        if(!Meteor.user())
            return Utils.notLoggedInErrorMsg;

        var username = Meteor.user().username;
        
        var cursor = UserProfiles.findOne({username: username});

        if (cursor === null || typeof(cursor) === 'undefined')
            return 'No profile found for ' + username + '.';

        var myPubKey = Trustore.Keys.getAsymPublic();
        var profileId = Meteor.user().profile.profileId;
        //var profileKey = Trustore.Keys.get('profile');
        var trustedHash = (Acls.getAcl(profileId, username, myPubKey)).hash;
        var profileKeys = Acls.getContentKeys(profileId, username, myPubKey, username, Trustore.Keys.get('master'), trustedHash);

        var msg = EJSON.stringify({username: cursor.username, aclVer: cursor.aclVer, profile: cursor.profile, timeStamp: cursor.timeStamp});

        if (!CryptoUtils.verifySig(myPubKey, msg, cursor.signature))
            throw 'Profile signature verification failed.';

        console.log('Decrypting own aclVer ' + cursor.aclVer +'.');

        return CryptoUtils.decrypt(profileKeys[cursor.aclVer-1], cursor.profile.text);
    },

    othersProfile: function() {
        if(!Meteor.user())
            return Utils.notLoggedInErrorMsg;

        var username = Session.get("otherProfileUsername");
        if (username === Meteor.user().username)
            return 'But your profile is just above this!';
        
        var otherUser = Meteor.users.findOne({username: username});
        if (otherUser === null || typeof(otherUser) === 'undefined')
            return 'No such user!';
        
        var cursor = UserProfiles.findOne({username: username});

        if (cursor === null || typeof(cursor) === 'undefined')
            return 'No profile found for ' + username + '.';

        var otherUserPubKey = CryptoUtils.unserializePublicKey(otherUser.profile.publicKey);
        var profileId = otherUser.profile.profileId;
        var friendKey = Friends.getFriendKey(Meteor.user().username, Trustore.Keys.get('master'), username);
        var trustedHash = (Acls.getAcl(profileId, username, otherUserPubKey)).hash;
        var profileKeys = Acls.getContentKeys(profileId, username, otherUserPubKey, Meteor.user().username, friendKey, trustedHash);

        var msg = EJSON.stringify({username: cursor.username, aclVer: cursor.aclVer, profile: cursor.profile, timeStamp: cursor.timeStamp});

        if (!CryptoUtils.verifySig(otherUserPubKey, msg, cursor.signature))
            throw 'Profile signature verification failed.';

        if (profileKeys.length < cursor.aclVer)
            throw 'ACL is older than profile. Something bad has happened.';

        console.log('Decrypting ' + username + '\'s aclVer ' + cursor.aclVer +'.');

        return CryptoUtils.decrypt(profileKeys[cursor.aclVer-1], cursor.profile.text);/**/ return 'Place holder';
    }
});

Template.profile.events({
    "submit .update-profile" : function(event) {
        var newProfile = event.target.profile.value;

        var username = Meteor.user().username;
        var myPubKey = Trustore.Keys.getAsymPublic();
        var profileId = Meteor.user().profile.profileId;
        var currentAcl = Acls.getAcl(profileId, username, myPubKey);
        var profileKeys = Acls.getContentKeys(profileId, username, myPubKey, username, Trustore.Keys.get('master'), currentAcl.hash);

        var aclVer = currentAcl.acl.aclVer;
        console.log('Encrypting to aclVer ' + aclVer +'.');
        var profile = {
            text: CryptoUtils.encrypt(profileKeys[aclVer-1], newProfile)
        };

        var timeStamp = new Date();
        
        var msg = EJSON.stringify({username: username, aclVer: aclVer, profile: profile, timeStamp: timeStamp});
        var signature = CryptoUtils.signString(Trustore.Keys.getAsymPrivate(), msg);

        //Meteor.call('updateProfile', profileObject);
        UserProfiles.update(profileId, {$set: {aclVer: aclVer, profile: profile, timeStamp: timeStamp, signature: signature}});
        
        Trustore.Keys.set('profile', profileKeys[aclVer-1]);

        event.target.profile.value = "";
        return false;
    },

    "submit .view-profile" : function(event) {
        var username = event.target.username.value;
        Session.set("otherProfileUsername", username);

        event.target.username.value = "";
        return false;
    }
});
        

