Quack = function(q, text) {
  this.log = function (msg) {
    console.log('[Quack] ' + msg);
  };

  this.data = q;
  if(text)
    this.text = text;
}

Quack.prototype.getPrevQuack = function() {
  var q = Quacks.findOne({ _id: this.data.prevQuackId });
  if(q)
    return new Quack(q);
  else {
    var d = Dialogues.findOne({ _id: this.data.prevQuackId });
    if(d)
      return new Dialogue(d);
    else
      return null;
  }
}

Quack.prototype.getPrevHash = function() {
  return this.data.prevQuackHash;
}

Quack.prototype.hash = function() {
  assert(this.data != null);
  var hash = Quack.hashData(this.data);
  return hash;
}

Quack.prototype.getText = function() { return this.text; }
Quack.prototype.sign = function(privKey) {
  var hash = this.hash();
  this.data.signature = CryptoUtils.signString(privKey, hash);
}

Quack.prototype.insert = function() {
  this.data._id = Quacks.insert(this.data);
}

Quack.create = function (type, author, aclVer, text, prevQuack) {
  assert(aclVer > 0);
  var data = {};
  
  data.type = type; // wall post, comment or message
  data.date = Utils.getDate(); // the date from Utils.getDate()
  data.author = author;
  data.encMsg = null;     // the encrypted quack (i.e. the message) under the key of parent object for the specified version in aclVer
  data.signature = null;  // the signature over this quack by its author
  data.aclVer = aclVer; // the ACL version of the parent object this quack is part of
  data.prevQuackHash = prevQuack.hash();  // the hash of the previous quack
  data.prevQuackId = prevQuack.data._id;

  return new Quack(data, text);
}

Quack.get = function(quackId) {
  var q = Quacks.findOne({ _id: quackId });
  return new Quack(q);
}

Quack.hashData = function(data) {
  var str = 'quack|' + data.type + '|' + data.author + '|' + data.date + '|' 
    + data.aclVer + '|' + data.encMsg + '|' + data.prevQuackHash;
  //this.log('Hashin\' quack: ' + str);
  
  return CryptoUtils.hash(str);
}

Dialogue = function (d) {
  this.log = function (msg) {
    console.log('[Dialogue] ' + msg);
  };

  this.data = d;
}

Dialogue.prototype.insert = function() {
  assert(!this.data._id);

  if(this.data.type == 'wall')
    assert(Dialogues.find({ type: this.data.type, owner: this.data.owner }).count() == 0);

  this.data._id = Dialogues.insert(this.data);

  if(this.data.type == 'wall')
    assert(Dialogues.find({ type: this.data.type, owner: this.data.owner }).count() == 1);
  
  this.log('Inserted dialogue: ' + this.toString());
}

Dialogue.prototype.getLastQuack = function() {
  // What if there's no last quack? return null!
  //this.log('Last quack ID for dialogue ' + this.data._id + ': ' + this.data.lastQuackId);
  var q = Quacks.findOne({ _id: this.data.lastQuackId });
  if(!q)
    return null;

  q = new Quack(q);

  return q;
}

Dialogue.prototype.hash = function() {
  return Dialogue.hashData(this.data);
}

Dialogue.prototype.addQuack = function(quack, contentKey, privKey) {
  var encMsg = CryptoUtils.encrypt(contentKey, quack.getText());

  quack.data.encMsg = encMsg;
  quack.sign(privKey);
  quack.insert();
  
  // update last quack in dialogue
  this.data.lastQuackId = quack.data._id;
  Dialogues.update({ _id: this.data._id }, { $set: { lastQuackId: quack.data._id }})
  //this.log('Added quack to dialogue: ' + stri(this.data));
}

Dialogue.prototype.toString = function() {
  return this.data._id + '|dialogue|' + this.data.type + '|' + this.data.owner 
    + '|' + this.data.title + '|lastQuackId=' + this.data.lastQuackId;
}

Dialogue.prototype.getObjectId = function() {
  return this.data.type + '|' + this.data.owner + '|' + this.data.rand;
}

Dialogue.hashData = function(data) {
  // SECURITY: The data._id is server generated, and so it does not help including
  // it in the hash to make the dialogue unique. we need to have it generated on the client
  // that's why we use data.rand
  var str = 'dialogue|' + data.rand + '|' + data.type + '|' + data.owner + '|' + data.title;
  //var str = 'dialogue|' + data.type + '|' + data.owner + '|' + data.title;
  var hash = CryptoUtils.hash(str);
  //console.log('Hashing dialogue ' + str + ': ' + hash);
  return hash;
}

Dialogue.getWall = function(username) {
  var d = Dialogues.findOne({ type: 'wall', owner: username });
  return new Dialogue(d);
}

Dialogue.prototype.removeUser = function(ownerMasterKey, ownerPrivateKey, theirUsername, 
  trustedAclHash)
{
  var objid = this.getObjectId();

  var c = UserConvos.findOne({ username: theirUsername });
  var did = this.data._id;
  var dialogues = c.dialogues.filter(function(d) { return d != did; });

  c = UserConvos.update({ _id: c._id }, { $set: { dialogues: dialogues  } });

  return Acls.removeFromAcl(objid, this.data.owner, ownerPrivateKey, ownerMasterKey, theirUsername,
    trustedAclHash);
}

// @param myMasterKey, the user who wants to query the dialogue for quacks will need to
// present his master keys which is used to fetch the public keys of the users who authored
// quacks
// @param trustedHash, the trusted hash of the last seen / cached quack. this can be the 
// hash of a quack or the hash of the dialogue itself.
Dialogue.prototype.getQuacks = function(me, myMasterKey, contentKeys, lastQuack, trustedHash) {
  // FIXME: pagination?
  var owner = this.data.owner;
  var dialogueId = this.getObjectId();
  
  check(me, String);
  check(dialogueId, String);
  check(trustedHash, String);
  assert(contentKeys);

  //console.log('Starting w/ last quack: ' + stri(lastQuack));

  if(!lastQuack) {
    Trustore.Hashes.set(dialogueId, trustedHash, me);
    return [];
  }

  // FIXME: PERFORMANCE/SECURITY: we download all quacks here when we don't have 
  // a trusted hash
  var quacks = [];
  var curHash = lastQuack.hash();
  var prevHash = lastQuack.getPrevHash();
  var quack = lastQuack;
  var prevQuack = lastQuack.getPrevQuack();
  var numDisplay = 25;
  var isVerified = false;

  //console.log(' and w/ prev quack: ' + stri(prevQuack));

  // This has to check for link to trustedLastQuack hash but it has to keep getting
  // enough quacks to display. Also, it has to update the trusted hash to lastQuack.hash()
  // once it's verified.
  while(prevQuack) {
    //console.log('Getting ' + quack.data.author + '\'s public key');
    var publicKey = Friends.getVouchedPublicKey(me, myMasterKey, owner, quack.data.author);
    assert(publicKey);
    //console.log('Public key: ' + CryptoUtils.serializePublicKey(publicKey));

    if(!CryptoUtils.verifySig(publicKey, curHash, quack.data.signature)) {
      throw new Meteor.Error('bad-quack-signature', 'Signature on quack hash ' + curHash
        + ' did not verify');
    }

    if(curHash == trustedHash) {
      Trustore.Hashes.set(dialogueId, curHash, me);
      isVerified = true;
    }

    var hash = prevQuack.hash();
    if(hash != prevHash) {
      throw new Meteor.Error('bad-quack-hash', 'Expected previous quack to have hash ' 
        + prevHash + ' but got hash ' + hash);
    }

    //console.log('quack aclVer: ' + quack.data.aclVer + ', content keys = ' + stri(contentKeys));

    // Append to quacks array
    if(numDisplay > 0) {
      //console.log('Pushing quack ' + quack.data._id);
      quacks.push({
        type: quack.data.type,
        date: quack.data.date,
        author: quack.data.author,
        aclVer: quack.data.aclVer,
        msg: CryptoUtils.decrypt(contentKeys[quack.data.aclVer - 1], quack.data.encMsg),
      });
    } else {
      // Just verifying the quack, but will not display it.
    }

    curHash = prevHash;
    // prevQuack can be a Dialogue here and these cannot be called, so we need to exit
    //console.log('Quack ID: ' + quack.data._id);
    //console.log('Prev quack ID: ' + prevQuack.data._id);
    //console.log('Dialogue ID: ' + this.data._id);
    if(prevQuack.data._id != this.data._id) {
      quack = prevQuack;
      prevHash = quack.getPrevHash();
      prevQuack = quack.getPrevQuack();
    } else {
      //console.log('Reached end of dialogue!');
      break;
    }
    numDisplay--;

    // If we've verified the quacks by linking them to the old trusted hash
    // then we can exit this loop
    if(isVerified && numDisplay < 0) {
      //console.log('Reached last trusted quack and reached target # of quacks');
      break;
    }
  }

  // At this point, we reached the end of the quack chain and have
  // not linked back to any quack. It means we should be able to link
  // back to the dialogue.
  // prevHash has the hash of the dialogue and we can check it
  // against the dialogue
  if(prevQuack == null) {
    assert(prevHash);

    if(trustedHash != prevHash) {
      throw new Meteor.Error('quack-did-not-link', 'The quack chain did not properly link back '
        + 'to the dialogue. Dialogue hash was ' + trustedHash 
        + ' vs. quack\'s prev. hash ' + prevHash);
    }

    Trustore.Hashes.set(dialogueId, trustedHash, me);
  }

  //console.log('Returning ' + quacks.length + ' quacks');
 
  return quacks;
}

Dialogue.createWall = function(username) {
  var d = { 
    type: 'wall', 
    owner: username, 
    title: username + '\'s wall',
    lastQuackId: null,
    rand: CryptoUtils.randomSalt(),
  };
  
  var dialogue = new Dialogue(d);

  dialogue.insert();

  return dialogue;
}

Dialogue.getConversationIDs = function(memberName) {
  var found = UserConvos.findOne({ username: memberName });
  if(found) {
    return found.dialogues;
  } else {
    console.log('No convo found for ' + memberName);
    return [];
  }
}

Dialogue.getConversation = function(id) {
  return new Dialogue(Dialogues.findOne({ type: 'convo', _id: id }));
}

Dialogue.createConversation = function(owner, ownerMasterKey, ownerPrivateKey, withUsers, title) {
  var members = _.pluck(withUsers, 'username');
  
  console.log(owner + ' w/ master key' + CryptoUtils.serializeSymKey(ownerMasterKey)
    + ' and key pair ' + CryptoUtils.serializePrivateKey(ownerPrivateKey)
    + ' is writing to ' 
    + _.reduceRight(members, function(a, b) { return a + ', ' + b; }));

  var d = {
    type: 'convo',
    owner: owner,
    title: title,
    date: new Date(),
    lastQuackId: null,
    rand: CryptoUtils.randomSalt(),
  };

  var convo = new Dialogue(d);

  convo.insert();
  var c = UserConvos.findOne({ username: owner });
  c = UserConvos.update({ _id: c._id }, { $push: { dialogues: convo.data._id  } });
  console.log('Added ' + convo.data._id + ' (c = ' + c + ') to ' + owner + '\'s conversations!');

  var acl = Acls.createAcl(convo.getObjectId(), owner, ownerMasterKey, ownerPrivateKey);
  var contentKey = acl.contentKey;
  var trustedHash = acl.hash;

  for(var i = 0; i < withUsers.length; i++) {
    var member = withUsers[i].username;
    var c = UserConvos.findOne({ username: member });
    c = UserConvos.update({ _id: c._id }, { $push: { dialogues: convo.data._id  } });
    console.log('Added ' + convo.data._id + ' (c = ' + c + ') to ' + member + '\'s conversations!');
    trustedHash = Acls.addToAcl(convo.getObjectId(), owner, ownerPrivateKey, ownerMasterKey, 
      member, trustedHash);
  }

  return { dialogue: convo, contentKey: contentKey, aclHash: trustedHash };
}

