Friends = {
  // @param myUsername your username as the initiator of the friendship request
  // @param theirUsername the username of the other party whom you are adding as a friend
  addFriend: function (myUsername, myPrivateKey, theirUsername, theirPublicKey) {
    // Add the other party to your friend list
    var r = Friends.addFriendHelper(myUsername, myPrivateKey, theirUsername, theirPublicKey, true);
    
    // Send the other party's a friend request
    var req = Friends.sendFriendRequestHelper(myUsername, myPrivateKey, theirUsername, theirPublicKey);

    // TODO: Add the other party to your ACL
    //Acl.write(objid, myUsername, myPrivateKey, theirUsername);

    r.date = req.date;
    r.reqSig = req.reqSig;
    return r;
  },

  // @param myUsername your username as the user who's been friend-requested
  // @param theirUsername the username of the person who sent the friend request
  // @param date the date at which the request was sent
  // @param signature by the friend request sender on the request
  acceptFriend: function (myUsername, myPrivateKey, theirUsername, theirPublicKey, date, reqSig) {
    var r = Friends.addFriendHelper(myUsername, myPrivateKey, theirUsername, theirPublicKey, false);

    // Verify signature
    var msg = 'friendRequest|' + theirUsername + '|' + myUsername + '|' + date;
    if(CryptoUtils.verifySig(theirPublicKey, msg, reqSig) == false)
      throw 'ERROR: Signature by \'' + theirUsername + '\' on friend request \'' + msg + '\' did not verify. Something is fishy.';

    // If all is well, remove FriendRequest row
    //console.log(myUsername + '\'s old friend requests: ' + EJSON.stringify(Friends.getFriendRequests(myUsername)));
    Meteor.users.update({ _id: Utils.getUserId(myUsername) }, {
     $pull: { 'profile.friendRequests': { initiator: theirUsername, date: date, signature: reqSig } }
    });
    //console.log(myUsername + '\'s new friend requests: ' + EJSON.stringify(Friends.getFriendRequests(myUsername)));

    // TODO: Add new friend to your ACL
    //Acl.write(objid, myUsername, myPrivateKey, theirUsername);
    return r;
  },

  addFriendHelper: function (myUsername, myPrivateKey, theirUsername, theirPublicKey, direction) {
    check(direction, Boolean);
    
    if(Friends.isMyFriend(myUsername, theirUsername))
      throw theirUsername + ' is already friends with you';

    var initiator = direction ? myUsername : theirUsername;
    var target = direction ? theirUsername : myUsername;
    
    var ecdhKey = CryptoUtils.ecdhDeriveKey(myPrivateKey, theirPublicKey);
    // We need to account for the direction in which the friendship was initiated when we
    // derive the friend key
    var friendKey = Utils.deriveFriendKey(initiator, target, ecdhKey);
    var masterKey = Trustore.Keys.get('master', myUsername);
    // FIXME: SECURITY: need to also bind the MAC tag to the friend's username, otherwise server can 
    // give us another friend's friend key
    var encFriendKey = CryptoUtils.encryptKey(masterKey, friendKey);
    var theirPublicKeyStr = CryptoUtils.serializePublicKey(theirPublicKey);
    // FIXME: SECURITY: need to also bind this MAC to the friend's username, otherwise server can give
    // us another freind's public key
    var macTheirPublicKey = CryptoUtils.mac(masterKey, theirPublicKeyStr);

    // NOTE: already tested that both parties derive the same key!
    //console.log('derived friend key between ' + myUsername + ' and ' + theirUsername + ': ' + friendKey);
    //console.log('enc friend key: ' + EJSON.stringify(encFriendKey));
    //console.log('their public key MAC: ' + macTheirPublicKey);

    // Store Friend w/ friendKey in user.profile.friends[i]
    // NOTE: Here we use myUsername/theirUsername and not initator/target for a reason:
    // we have to update the other user's profile with his new friend
    var myId = Utils.getUserId(myUsername);
    //console.log(myUsername + ' is adding friend key '
    //  + CryptoUtils.serializeSymKey(friendKey) + ' for ' + theirUsername
    //  + ', encrypted as: ' + EJSON.stringify(encFriendKey));
    var newFriend = {
      username: theirUsername,
      publicKey: theirPublicKeyStr,
      macPublicKey: macTheirPublicKey,
      encAuthFriendKey: encFriendKey, 
    };

    // Q: SECURITY: Can user `u` update user `w`'s profile? 
    // A. No. Meteor prevents that from happening as it should.
    //console.log('before friends: ' + EJSON.stringify(Friends.getFriends(myUsername)));
    Meteor.users.update({ _id: myId }, { $push: { 'profile.friends': newFriend }});
    //console.log(myUsername + '\'s new friend list: ' 
    //  + EJSON.stringify(Friends.getFriends(myUsername)));

    //var profileId = Meteor.user().profile.profileId;
    // FIXME: Profile ID should be profile|username
    var profileId = Utils.getProfileId(myUsername);
    var albumIds = Utils.getAlbumIds(myUsername);
    //console.log('profileId: ' + profileId);
    var trustedHash = (Acls.getAcl(profileId, myUsername, Trustore.Keys.getAsymPublic(myUsername))).hash;

    Acls.addToAcl(profileId, myUsername, myPrivateKey, masterKey, theirUsername, 
      Trustore.Hashes.get('acl.' + profileId, myUsername));
    
    var wallId = Dialogue.getWall(myUsername).getObjectId();
    var wallAclHash = Trustore.Hashes.get('acl.' + wallId, myUsername);
    if(!wallAclHash)
      wallAclHash = Acls.getAcl(wallId, myUsername, Trustore.Keys.getAsymPublic(myUsername)).hash;
    console.log('wall ID for ' + myUsername + ': ' + wallId);
    Acls.addToAcl(wallId, myUsername, myPrivateKey, masterKey, 
      theirUsername, wallAclHash);

    for (i = 0; i < albumIds.length; i++)
        Acls.addToAcl(albumIds[i], myUsername, myPrivateKey, masterKey, theirUsername, 
          Trustore.Hashes.get('acl.' + albumIds[i], myUsername));


    return {
      friendKey: friendKey
    };
  },

  sendFriendRequestHelper: function (myUsername, myPrivateKey, theirUsername, theirPublicKey) {
    var date = Utils.getDate();
    var msg = 'friendRequest|' + myUsername + '|' + theirUsername + '|' + date;
    var signature = CryptoUtils.signString(myPrivateKey, msg);

    // FIXME: This is asynchronous, so that means when it returns the data has not necessarily
    // been set in the DB. Be careful when you test this function and make assumptions about the 
    // DB.
    Meteor.call('Friends_sendRequest', myUsername, theirUsername, date, signature, function(err, result) {
      // If the method throws an exception, it will be stored in 'err'. Otherwise 'err' will
      // be undefined. Example:
      // err: {"error":"friend-request-already-sent","reason":"You've already sent a friend request to prashant",
      //"message":"You've already sent a friend request to prashant [friend-request-already-sent]","errorType":"Meteor.Error"}
      //console.log('err: ' + EJSON.stringify(err));
      
      // 'result' will store the method's return value, if any. Otherwise, it will be undefined.
      //console.log('result: ' + EJSON.stringify(result));
    });

    return {
      date: date,
      reqSig: signature,
    }
  },

  removeFriend: function (username) {
    // TODO: implement, must remove user from all acls
  },

  // FIXME: PERFORMANCE: This is slow. If we have a Friends table we can maybe index in it.
  isMyFriend: function (myUsername, theirUsername) {
    check(theirUsername, String);
    check(myUsername, String);
    var friends = Friends.getFriends(myUsername);
    //console.log('friends: ' + friends);
    return _.contains(friends, theirUsername);
  },

  // NOTE: This call is directional! It can happen that f(a, b) != f(b, a)
  //
  // Returns true if 'myUsername' sent a friend request to 'theirUsername' and
  // 'theirUsername' has not accepted it yet. Returns false if they are friends.
  // Returns false if they are not friends and no request has been sent.
  // 
  // @param myUsername the person who is asking if he sent the friend request
  // @param theirUsername the person whom they are asking about
  //
  // @return true or false (see above)
  isFriendRequestSent: function(myUsername, theirUsername) {
    // Meteor does not support $elemMatch to only return a single friendRequest sub-document,
    // but in this case, we just need the confirmation that there exists a friend request, so
    // the fact that all requests are returned doesn't affect correctness, only performance (yay!)
    var r = Meteor.users.findOne({ _id: Utils.getUserId(theirUsername), 'profile.friendRequests.initiator': myUsername }, { fields: { username: 1 }});
    //console.log('isFrReqSent(' + myUsername + ', ' + theirUsername + '): ' + EJSON.stringify(r, { indent: true }));

    return r ? true : false;
  },

  // @return returns an array of friend usernames like ['alin', 'bob', 'carl']
  getFriends: function (username) {
    var friends = _.pluck(Meteor.users.findOne({ _id: Utils.getUserId(username) }, { fields: { 'profile.friends.username': 1 }}).profile.friends, 'username');

    // Remove the people whom you've sent a friend request to
    // and have not accepted yet.
    friends = friends.filter(function(friend) {
      var r = Friends.isFriendRequestSent(username, friend);
      //console.log(r ? username + ' already sent request to ' + friend : username + ' did not send a request to ' + friend + ' yet');
      return !r;
    });

    return friends;
  },

  getFriendRequests: function (username) {
    check(username, String);
    var reqs = Meteor.users.findOne({ _id: Utils.getUserId(username) }, { fields: {
      'profile.friendRequests.initiator': 1,
      'profile.friendRequests.date': 1,
      'profile.friendRequests.signature': 1
    }}).profile.friendRequests;
    //console.log('reqs from db: ' + EJSON.stringify(reqs));
    return reqs;
  },
  
  // 'asker' is trying to get 'target's' public key via 'intermediary'
  // Check all cases and return validated public key.
  // It could be that asker = intermediary = target, so just return your
  // own public key in that case.
  getVouchedPublicKey: function (asker, askerMasterKey, intermediary, target) {
    // TODO: might run into problems here, so might need to add another 'me' parameter
    if(asker == target /*&& asker == Utils.me()*/) {
      return Trustore.Keys.getAsymPublic(asker);
    }

    if(asker != target) {
      if(Friends.isMyFriend(asker, target)) {
        return Friends.getFriendPublicKey(asker, askerMasterKey, target);
      } else {
        // FIXME: should implement getting vouched public keys here
        return CryptoUtils.unserializePublicKey(Utils.getPublicKeyByUsername(target));
      }
    } else {
      // TODO: not clear if we'll ever call this with asker = target but != Utils.me()
      // maybe we will in the server code.
      return CryptoUtils.unserializePublicKey(Utils.getPublicKeyByUsername(target));
    }
  },

  getFriendPublicKey: function (myUsername, myMasterKey, theirUsername) {
    check(myUsername, String);
    assert(myMasterKey);
    check(theirUsername, String);
    
    console.log(myUsername + ' is looking for ' + theirUsername 
      + '\'s public key');

    // Meteor does not support MongoDB's $elemMatch, so we return all friends here
    // (yeah, that's right... JavaScript coding) and do a linear search for theirUsername
    var userEntry = Meteor.users.findOne(
      { _id: Utils.getUserId(myUsername), 'profile.friends.username': theirUsername },
      { fields: { 'profile.friends.username': 1, 'profile.friends.publicKey': 1, 'profile.friends.macPublicKey': 1 }});

    if (userEntry === null || typeof(userEntry) === 'undefined')
        throw 'We aren\'t friends. In fact, we barely know each other.';

    var friends = userEntry.profile.friends;

    var friend = _.find(friends, function(item) {
      return item.username == theirUsername;
    });
    var publicKey = friend.publicKey;
    var macPublicKey = friend.macPublicKey;
    
    //console.log('friend: ' + EJSON.stringify(friend));
    //console.log('myMasterKey: ' + CryptoUtils.serializeSymKey(myMasterKey));
    //console.log(myUsername + '\'s encFriendKey for ' + theirUsername + ': ' + EJSON.stringify(encFriendKey));

    // FIXME: SECURITY: gotta check the username included in the MAC on the public key
    if(!CryptoUtils.verifyMac(myMasterKey, friend.publicKey, friend.macPublicKey))
      throw new Meteor.Error('mac-integrity-check', 'Could not verify MAC on ' + theirUsername 
        + '\'s public key. The alleged MAC was: ' + friend.macPublicKey);
  
    return CryptoUtils.unserializePublicKey(friend.publicKey);
  },
  
  getFriendKey: function (myUsername, myMasterKey, theirUsername) {
    assert(myUsername != theirUsername, 'you are not your own friend');
    check(myUsername, String);
    check(theirUsername, String);
    assert(myMasterKey);
    // Meteor does not support MongoDB's $elemMatch, so we return all friends here
    // (yeah, that's right... JavaScript coding) and do a linear search for theirUsername
    var friends = Meteor.users.findOne(
      { _id: Utils.getUserId(myUsername), 'profile.friends.username': theirUsername },
      { fields: { 'profile.friends.username': 1, 'profile.friends.encAuthFriendKey': 1 }}).profile.friends;

    var friend = _.find(friends, function(item) {
      return item.username == theirUsername;
    });
    var encFriendKey = friend.encAuthFriendKey;
    
    //console.log('friend: ' + EJSON.stringify(friend));
    //console.log('myMasterKey: ' + CryptoUtils.serializeSymKey(myMasterKey));
    //console.log(myUsername + '\'s encFriendKey for ' + theirUsername + ': ' + EJSON.stringify(encFriendKey));

    // FIXME: SECURITY: gotta check the username included in the MAC on the friendkey
    return CryptoUtils.decryptKey(myMasterKey, encFriendKey);
  },
}

Meteor.methods({
  Friends_sendRequest: function (myUsername, theirUsername, date, signature) {
    if(Friends.isFriendRequestSent(myUsername, theirUsername))
      throw new Meteor.Error('friend-request-already-sent', 'You\'ve already sent a friend request to ' + theirUsername);

    var theirId = Utils.getUserId(theirUsername);
    Meteor.users.update({ _id: theirId }, { $push: { 'profile.friendRequests': {
      initiator: myUsername,
      date: date,
      signature: signature
    }}});

    return true;
  },
});
