Trustore = {
  Keys: {

    get: function (objid, username) {
      if(!username)
        username = Meteor.user().username;
      console.log('[Trustore] getting ' + username + '\'s key for \'' + objid + '\'');
      return EJSON.parse(localStorage[username + '.key.' + objid]);
    },

    set: function (objid, key, username) {
      if(!username)
        username = Meteor.user().username;

      console.log('[Trustore] setting ' + username + '\'s key for \'' + objid + '\'');
      localStorage[username + '.key.' + objid] = EJSON.stringify(key, { canonical: true });
    },

    clear: function (username) {
      if(!username)
        username = Meteor.user().username;

      console.log('[Trustore] clearing all of ' + username + '\'s keys');
      for(var key in localStorage) {
        if(key.indexOf(username + '.key.') == 0) {
          localStorage.removeItem(key);
          console.log('[Trustore] cleared \'' + key + '\'');
        }
      }
    },

    setAsym: function(keyPair, username) {
      if(!username)
        username = Meteor.user().username;

      var keyPairHex = {
        priv: CryptoUtils.serializePrivateKey(keyPair),
        pub: CryptoUtils.serializePublicKey(keyPair)
      };

      console.log('[Trustore] setting ' + username + '\'s asymmetric keypair');// + EJSON.stringify(keyPairHex));
      localStorage[username + '.key.asym'] = EJSON.stringify(keyPairHex, { canonical: true });
    },

    // @return an actual KeyPair object with the private and public key
    getAsym: function(username) {
      if(!username)
        username = Meteor.user().username;

      var keyPairHex = EJSON.parse(localStorage[username + '.key.asym']);

      console.log('[Trustore] getting ' + username + '\'s asymmetric keypair');// + EJSON.stringify(keyPairHex));
      return keyPairHex;
    },

    getAsymPrivate: function(username) {
      if(!username)
        username = Meteor.user().username;

      var keyPairHex = EJSON.parse(localStorage[username + '.key.asym']);

      console.log('[Trustore] getting ' + username + '\'s private key');// + EJSON.stringify(keyPairHex));
      return CryptoUtils.unserializePrivateKey(keyPairHex.priv);
    },

    getAsymPublic: function(username) {
      if(!username)
        username = Meteor.user().username;

      var keyPairHex = EJSON.parse(localStorage[username + '.key.asym']);

      console.log('[Trustore] getting ' + username + '\'s public key');// + EJSON.stringify(keyPairHex));
      return CryptoUtils.unserializePublicKey(keyPairHex.pub);
    }
  },

  Hashes: {
    // @return the trusted hash for that object ID, if any was set
    get: function(objid, username) {
      check(objid, String);
      if(!username)
        username = Meteor.user().username;

      var h = localStorage[username + '.hash.' + objid];
      console.log('[Trustore] getting ' + username + '\'s trusted hash for \'' 
        + objid + '\': ' + h);
      check(h, Match.OneOf(String, null, undefined));
      return h;
    },

    set: function(objid, trustedHash, username) {
      check(objid, String);
      check(trustedHash, String);

      if(!username)
        username = Meteor.user().username;

      console.log('[Trustore] setting ' + username + '\'s trusted hash for \'' 
        + objid + '\' to: ' + trustedHash);
      check(trustedHash, String);
      localStorage[username + '.hash.' + objid] = trustedHash;
    },

    clear: function (username) {
      if(!username)
        username = Meteor.user().username;

      console.log('[Trustore] clearing all of ' + username + '\'s hashes');
      for(var key in localStorage) {
        if(key.indexOf(username + '.hash.') == 0) {
          localStorage.removeItem(key);
          console.log('[Trustore] cleared \'' + key + '\'');
        }
      }
    },
  }
}
