str = function(o) {
  return EJSON.stringify(o);
}

stri = function(o) {
  return EJSON.stringify(o, { indent: true, canonical: true });
}

Utils = {
  me: function() {
    return Meteor.user().username;
  },

  getUserId: function (username) {
    return Meteor.users.findOne({ username: username }, { fields: { _id: 1 }})._id;
  },

  getProfileId: function (username) {
    return Meteor.users.findOne({ username: username }, { fields: { 'profile.profileId': 1 }}).profile.profileId;
  },

  getAlbumIds: function (username) {
    return Meteor.users.findOne({ username: username }, { fields: { 'profile.albumIds': 1 }}).profile.albumIds;
  },

  getUserName: function (userId) {
    return Meteor.users.findOne({ _id: userId }, { fields: { username: 1 }}).username;
  },

  getDate: function() {
    return new Date().toUTCString();
  },

  deriveMasterKey: function (password, salt) {
    //return sjcl.misc.pbkdf2(password, salt, 100000, 256);
    return CryptoUtils.keyDerivationFunction(password, salt);
  },

  deriveFriendKey: function (initiatorUser, targetUser, ecdhKey) {
    return CryptoUtils.keyDerivationFunction(ecdhKey, initiatorUser + '|' + targetUser);
  },

  createEmptyProfile: function(username, asymKeyPair, masterKey) {
    // Creates an empty profile for the specified user and also creates an ACL for it, generating the required content key in the process.
    // Returns {objid of the profile, content key for the profile, hash of the ACL tree}.
    var privKey = CryptoUtils.unserializePrivateKey(asymKeyPair.priv);
    var profileKey = CryptoUtils.genSymKey();

    var aclVer = 1;
    var profile = {
        text: CryptoUtils.encrypt(profileKey, "<empty profile>")
    };

    var timeStamp = new Date();

    var msg = EJSON.stringify({username: username, aclVer: aclVer, profile: profile, timeStamp: timeStamp});
    var signature = CryptoUtils.signString(privKey, msg);

    var profileObject = {
        username: username,
        aclVer: aclVer,
        profile: profile,
        timeStamp: timeStamp,
        signature: signature
    };

    //console.log(profileObject);
    var objid = UserProfiles.insert(profileObject);
    UserProfiles.update(objid, {$set: {objid: objid}});

    // FIXME: i think objid should be 'profile|username'
    var Acl = Acls.createAcl(objid, username, masterKey, privKey, profileKey);
    
    return {profileId: objid, profileKey: profileKey, profileAclTree: Acl};
  },

  createEmptyWall: function(username, asymKeyPair, masterKey) {
    assert(Dialogues.find({ owner: username }).count() == 0);
    
    var wall = Dialogue.createWall(username);

    var acl = Acls.createAcl(wall.getObjectId(), username, masterKey, asymKeyPair);
    //console.log('Acl for wall: ' + stri(acl));
  },

  findUsersRegex: function(me, regex) {
    return Meteor.users.find(
      { $and: [{ username: { $regex: regex } }, { username: { $ne: me }}]}, 
      { fields: { username: 1, 'profile.publicKey': 1 }, sort: { username: 1 } }
    ).fetch();
  },

  // @param callback gets called after Accounts.createUser completes
  // and indicates success or failure. It needs to be of the following type:
  // function callback(hasError, creds) {
  // }
  // See code below for more.
  registerAccount: function (username, email, password, callback) {
    var masterKeySalt = CryptoUtils.randomSalt();
    var asymKeyPair = CryptoUtils.genAsymKeyPair();
    var pubKeyStr = CryptoUtils.serializePublicKey(asymKeyPair);
    var masterKey = Utils.deriveMasterKey(password, masterKeySalt);

    var encAuthKeyPair = {
      priv: CryptoUtils.encrypt(masterKey, CryptoUtils.serializePrivateKey(asymKeyPair)),
      pub: CryptoUtils.encrypt(masterKey, pubKeyStr)
    };

    var creds = {
      username: username,
      masterKeySalt: masterKeySalt,
      masterKey: masterKey,
      asymKeyPair: asymKeyPair,
    };

    // Create an empty profile for this user (along with empty ACL), store its key and hash in Trustore, and its objid in Meteor.users.
    var createdProfile = { profileId: null };
    if (Meteor.isServer) {
      createdProfile = Utils.createEmptyProfile(username, asymKeyPair, masterKey);
      creds.profile = createdProfile;
    }

    UserConvos.insert({ username: username, dialogues: [] });
    Utils.createEmptyWall(username, asymKeyPair, masterKey);
    
    Accounts.createUser({
      username: username,
      email: email,
      password: password,
      profile: {
        masterKeySalt: masterKeySalt,
        encAuthKeyPair: encAuthKeyPair,
        publicKey: pubKeyStr,
        profileId: createdProfile.profileId,
        albumIds: [],
      }   
    }, Meteor.isServer ? null : function(hasError) {
      var createdProfile = Utils.createEmptyProfile(username, asymKeyPair, masterKey);
      creds.profile = createdProfile;
      Meteor.users.update(Meteor.userId(), { $set: { 'profile.profileId': createdProfile.profileId }});
      callback(hasError, creds);
    });

    return creds;
  },

  // TODO: SECURITY: SERVERACL: Might not want to respond to all requests
  // for public keys
  //
  // @return a hex-encoded public key as a string
  getPublicKeyByUsername: function (username) {
    var user = Meteor.users.findOne({ username: username }, { fields: { 'profile.publicKey': 1 } });
    //console.log(username + '\'s public key: ' + EJSON.stringify(user));
    var pubKey = user.profile.publicKey;

    return pubKey;
  },

  // NOTE: not used yet, i used the CryptoUtils function directly when I needed it
  //
  // @return the fingerprint as a hex-encoded string
  getFingerprintByUsername: function (username) {
    var pubKey = Utils.getPublicKeyByUsername(username);

    var fingerprint = CryptoUtils.getPublicKeyFingerprint(pubKey);

    return fingerprint;
  },

  notLoggedInErrorMsg: 'Meteor is being silly and calling this when I\'m not logged in.',
}
