var log = function(msg) { console.log('[TestAcls] ' + msg); };

TestAcls = {
  run: function() {
    var r = TestAcls.init();

    var users = r.users;
    var friendKeys = r.friendKeys;
    var owner = r.owner;

    TestAcls.helpersTest();
    TestAcls.addTest(users, friendKeys, owner);
    TestAcls.getAclTest(users, friendKeys, owner);
    TestAcls.removeTest(users, friendKeys, owner);

    log('WOW, all done!');
  },

  removeAllAcls: function() {
    // Clean up after previous test
    AclTrees.remove({});
    AclNodes.remove({});
    AclIndexes.remove({});
  },

  init: function() { 

    TestUsers.removeAllUsers();

    var owner = TestUsers.createUser('ACL_ownerUser11');
    var users = [
       TestUsers.createUser('alice'),
       TestUsers.createUser('bob'),
       TestUsers.createUser('carl'),
       TestUsers.createUser('dan'),
       TestUsers.createUser('eve'),
       TestUsers.createUser('fiona'),
       TestUsers.createUser('gary'),
       TestUsers.createUser('harry'),
       TestUsers.createUser('ian'),
       //TestUsers.createUser('jesse'),
       //TestUsers.createUser('kevin'),
       //TestUsers.createUser('lamar'),
       //TestUsers.createUser('mary'),
       //TestUsers.createUser('nick'),
       //TestUsers.createUser('orpheus'),
       //TestUsers.createUser('patrick'),
       //TestUsers.createUser('quincy'),
       //TestUsers.createUser('randy'),
       //TestUsers.createUser('steven'),
       //TestUsers.createUser('tom'),
       //TestUsers.createUser('uyanga'),
       //TestUsers.createUser('veronica'),
       //TestUsers.createUser('will'),
       //TestUsers.createUser('xander'),
       //TestUsers.createUser('yannis'),
       //TestUsers.createUser('zola'),
    ];
    var friendKeys = [];

    for(var i = 0; i < users.length; i++)
       friendKeys.push(TestUsers.createFriendship(owner, users[i]));

    return { users: users, friendKeys: friendKeys, owner: owner };
  },

  addTest: function(users, friendKeys, owner) {
    // Create an ACL with a single user on it
    var objid = 'addTestObj';
    var contentKey = CryptoUtils.genSymKey();
    var tree = Acls.createAcl(objid, owner.username, owner.masterKey, owner.asymKeyPair, 
      contentKey, users[0].username, friendKeys[0]);
    var trustedHash = tree.hash;

    // Get the ACL content keys as the owner
    var contentKeysOwner = TestAcls.getContentKeys(owner.username, objid, owner, owner,
      owner.masterKey, trustedHash);

    // Get the ACL content keys as one of the users on them
    var contentKeys = TestAcls.getContentKeys('after adding people', objid, owner, users[0], 
      friendKeys[0], trustedHash);
    
    // Compare ACL content keys against each other
    assert.deepEqual(contentKeysOwner, contentKeys);
    assert.deepEqual([ contentKey ], contentKeys);

    for(var i = 1; i < users.length; i++) {
      trustedHash = Acls.addToAcl(objid, owner.username, owner.asymKeyPair, owner.masterKey,
        users[i].username, trustedHash);
   
      // Compare the content keys (as viewed by the owner) against the content keys
      // as viewed by all the users added so far.
      log('Checking content key after adding ' + users[i].username + ' ...');
      var ownCk = TestAcls.getContentKeys('owner\'s', objid, owner, 
        owner, owner.masterKey, trustedHash);

      for(var j = 0; j <= i; j++) {
        var ck = TestAcls.getContentKeys('after adding ' + users[j].username, objid, owner, 
          users[j], friendKeys[j], trustedHash);

        assert.deepEqual(ownCk, ck);
      }
    }
  },

  getContentKeys: function(atMsg, objid, owner, user, friendKey, trustedHash) {
    var contentKeys = Acls.getContentKeys(objid, owner.username, owner.asymKeyPair,
      user.username, friendKey, trustedHash);

    var contentKeysStr = _.map(contentKeys, function(key) { 
      return CryptoUtils.serializeSymKey(key);
    });

    //console.log('[TestAcls] contentKeys @ ' + atMsg + ': ' + EJSON.stringify(contentKeysStr));

    return contentKeys;
  },

  helpersTest: function() {
    assert(Acls.Helper.hasSpace({ numUsers: 0 }));
    assert(Acls.Helper.hasSpace({ numUsers: 1 }));
    assert(!Acls.Helper.hasSpace({ numUsers: 2 }));
    assert(Acls.Helper.hasSpace({ numUsers: 3 }));
    assert(!Acls.Helper.hasSpace({ numUsers: 4 }));
    assert(Acls.Helper.hasSpace({ numUsers: 5 }));
    assert(Acls.Helper.hasSpace({ numUsers: 6 }));
    assert(Acls.Helper.hasSpace({ numUsers: 7 }));
  },
  
  removeTest: function(users, friendKeys, owner) {
    // Create an ACL with a single user on it
    var objid = 'removeTestObj';
    var contentKey = CryptoUtils.genSymKey();
    var tree = Acls.createAcl(objid, owner.username, owner.masterKey, owner.asymKeyPair, 
      contentKey);
    var trustedHash = tree.hash;

    // Get the ACL content keys as the owner
    var contentKeysOwner = TestAcls.getContentKeys(owner.username, objid, owner, owner,
      owner.masterKey, trustedHash);

    // Compare ACL content keys against each other
    assert.deepEqual(contentKeysOwner, [ contentKey ]);

    // Add/remove stress test
    for(var i = 0; i < users.length; i++) {
      for(var c = 0; c <= 6; c++) {
        log('Adding ' + users[i].username);
        trustedHash = Acls.addToAcl(objid, owner.username, owner.asymKeyPair, owner.masterKey,
          users[i].username, trustedHash);

        log('Removing ' + users[i].username);
        trustedHash = Acls.removeFromAcl(objid, owner.username, owner.asymKeyPair, owner.masterKey,
          users[i].username, trustedHash);
      }
    }

    // Add & remove people from ACL
    for(var i = 0; i < users.length; i++) {
      log('Adding ' + users[i].username);
      trustedHash = Acls.addToAcl(objid, owner.username, owner.asymKeyPair, owner.masterKey,
        users[i].username, trustedHash);
   
      // Compare the content keys (as viewed by the owner) against the content keys
      // as viewed by the added user.
      log('Checking content key after adding ' + users[i].username + ' ...');
      var ownCk = TestAcls.getContentKeys('owner\'s', objid, owner, 
        owner, owner.masterKey, trustedHash);

      var ck = TestAcls.getContentKeys('after adding ' + users[i].username, objid, owner, 
        users[i], friendKeys[i], trustedHash);

      assert.deepEqual(ownCk, ck);

      log('Removing ' + users[i].username);
      trustedHash = Acls.removeFromAcl(objid, owner.username, owner.asymKeyPair, owner.masterKey,
        users[i].username, trustedHash);
    }

    // Add all people back
    for(var i = 0; i < users.length; i++) {
      log('Adding back ' + users[i].username);
      trustedHash = Acls.addToAcl(objid, owner.username, owner.asymKeyPair, owner.masterKey,
        users[i].username, trustedHash);
   
      // Compare the content keys (as viewed by the owner) against the content keys
      // as viewed by the added user.
      log('Checking content keys for everyone, after adding ' + users[i].username + ' ...');
      var ownCk = TestAcls.getContentKeys('owner\'s', objid, owner, 
        owner, owner.masterKey, trustedHash);

      for(var j = 0; j <= i; j++) {
        var ck = TestAcls.getContentKeys('For ' + users[j].username, objid, owner,
          users[j], friendKeys[j], trustedHash);

        assert.deepEqual(ownCk, ck);
      }
    }

    // Remove a few people
    for(var i = 0; i < users.length; i++) {
      if(i % 3) {
        log('Removing ' + users[i].username);
        trustedHash = Acls.removeFromAcl(objid, owner.username, owner.asymKeyPair, owner.masterKey,
          users[i].username, trustedHash);
      }

      var ownCk = TestAcls.getContentKeys('owner\'s', objid, owner, 
        owner, owner.masterKey, trustedHash);

      // Check that people who were not removed can still access the keys
      for(var j = 0; j < users.length; j++) {
        if(!(j % 3) || j > i) {
          var ck = TestAcls.getContentKeys('For ' + users[j].username, objid, owner,
            users[j], friendKeys[j], trustedHash);

          assert.deepEqual(ownCk, ck);
        }
      }
    }

    // Add them back
    for(var i = 0; i < users.length; i++) {
      if(i % 3) {
        log('Adding ' + users[i].username + ' back');
        trustedHash = Acls.addToAcl(objid, owner.username, owner.asymKeyPair, owner.masterKey,
          users[i].username, trustedHash);

        var ownCk = TestAcls.getContentKeys('owner\'s', objid, owner, 
          owner, owner.masterKey, trustedHash);

        // Check that people who were added back or were not removed can still access the keys
        for(var j = 0; j < users.length; j++) {
          if(j <= i || !(j % 3)) {
            var ck = TestAcls.getContentKeys('For ' + users[j].username, objid, owner,
              users[j], friendKeys[j], trustedHash);

            assert.deepEqual(ownCk, ck);
          }
        }
      }
    }
    return objid;
  },

  getAclTest: function(users, friendKeys, owner, objid) {
    // Create an ACL with a single user on it
    var objid = 'someTestObj';
    var tree = Acls.createAcl(objid, owner.username, owner.masterKey, owner.asymKeyPair);
    var trustedHash = tree.hash;

    // Add/remove a few users
    var numUsers = Math.floor(users.length / 2);
    var removeCount = 2;
    var versions = numUsers * (removeCount);

    for(var i = 0; i < numUsers; i++) {
      for(var c = 0; c < removeCount; c++) {
        log('Adding ' + users[i].username);
        trustedHash = Acls.addToAcl(objid, owner.username, owner.asymKeyPair, owner.masterKey,
          users[i].username, trustedHash);

        log('Removing ' + users[i].username);
        trustedHash = Acls.removeFromAcl(objid, owner.username, owner.asymKeyPair, owner.masterKey,
          users[i].username, trustedHash);
      }
    }

    var acl = Acls.getAcl(objid, owner.username, owner.asymKeyPair);
    console.log('ACL: ' + stri(acl));

    assert.equal(versions + 1, acl.acl.aclVer);
  },
}
