var log = function(msg) { console.log('[TestAlbums] ' + msg); };

TestMerkle = {
    run: function() {
        log('Testing Merkle trees.');
        user = TestUsers.createUser('merkleuser');
        log('Created user.');

        var treeId = MerkleTree.newTree('Some objid', 'merkleuser', user.asymKeyPair);
        log('Created tree ' + treeId);
        TestMerkle.printTree(treeId);

        var trustedHash = null;

        trustedHash = MerkleTree.insert(treeId, 'a', user.asymKeyPair, trustedHash);
        var nodeA = MerkleTree.indexLookUp(treeId, 'a');
        TestMerkle.printTree(treeId);

        trustedHash = MerkleTree.insert(treeId, 'b', user.asymKeyPair, trustedHash);
        var nodeB = MerkleTree.indexLookUp(treeId, 'b');
        TestMerkle.printTree(treeId);

        // Uncomment below to test verification.
        //MerkleTrees.update(treeId, {$push: {emptyLeafNodes: nodeB}});
        //MerkleTreeNodes.update(nodeB, {$set: {isEmpty: true, hash: []}});
        //MerkleTrees.update(treeId, {$set: {signature: "Please trust this signature."}});
        //MerkleTreeNodes.update(MerkleTrees.findOne(treeId).rootNode, {$set: {hash: "This is hash."}});
        //TestMerkle.printTree(treeId);

        trustedHash = MerkleTree.insert(treeId, 'c', user.asymKeyPair, trustedHash);
        var nodeC = MerkleTree.indexLookUp(treeId, 'c');
        TestMerkle.printTree(treeId);

        trustedHash = MerkleTree.insert(treeId, 'd', user.asymKeyPair, trustedHash);
        var nodeD = MerkleTree.indexLookUp(treeId, 'd');
        TestMerkle.printTree(treeId);

        trustedHash = MerkleTree.delete(treeId, nodeB, user.asymKeyPair, trustedHash);
        TestMerkle.printTree(treeId);

        trustedHash = MerkleTree.delete(treeId, nodeC, user.asymKeyPair, trustedHash);
        TestMerkle.printTree(treeId);

        trustedHash = MerkleTree.insert(treeId, 'e', user.asymKeyPair, trustedHash);
        var nodeE = MerkleTree.indexLookUp(treeId, 'e');
        TestMerkle.printTree(treeId);

    },

    printTree: function(merkletreeId) {
        var merkletree = MerkleTrees.findOne(merkletreeId);
        if (merkletree === null || typeof(merkletree) === 'undefined')
            throw 'Merkle tree does not exist.';
        
        var printNode = function(nodeId, indentLevel) {
            if (nodeId === null)
                return;

            var node = MerkleTreeNodes.findOne(nodeId);
            if (node === null || typeof(node) === 'undefined')
                throw 'Merkle tree has invalid node Id entry.';

            log(Array((indentLevel*4)+1).join(' ') + EJSON.stringify({_id: node._id, hash: node.hash, isEmpty: node.isEmpty}));

            printNode(node.leftChild, indentLevel+1);
            printNode(node.rightChild, indentLevel+1);
        }

        log('Printing tree ' + merkletreeId);
        log('objid: ' + EJSON.stringify(merkletree.objid));
        log('owner: ' + EJSON.stringify(merkletree.owner));
        log('signature: ' + EJSON.stringify(merkletree.signature));
        log('leafNodes: ' + EJSON.stringify(merkletree.leafNodes));
        log('emptyLeafNodes: ' + EJSON.stringify(merkletree.emptyLeafNodes));

        printNode(merkletree.rootNode, 0);

        log('');
    }

};

TestAlbums = {
    run: function() {
        log('Testing albums.');
        var username = 'albumuser'
        var user = TestUsers.createUser(username);

        var albumId0 = Album.createAlbum(username, user.masterKey, user.asymKeyPair, 'album0');
        var albumId = Album.createAlbum(username, user.masterKey, user.asymKeyPair, 'album1');
        log('Created albums.');

        var username2 = 'albumuser2'
        var user2 = TestUsers.createUser(username2);
        var username3 = 'albumuser3'
        var user3 = TestUsers.createUser(username3);
        log('Created users.');

        TestUsers.createFriendship(user, user2);
        TestUsers.createFriendship(user, user3);
        log('Established friendship.');

        var albumId2 = Album.createAlbum(username2, user2.masterKey, user2.asymKeyPair, 'album2');

        var trustedHash = null;
        var photo1 = Album.insertPhoto(albumId, username, user.masterKey, user.asymKeyPair, 'photo1', trustedHash).photoId;
        //TestAlbums.printAlbum(albumId, username, user);
        var photo2 = Album.insertPhoto(albumId, username, user.masterKey, user.asymKeyPair, 'photo2', trustedHash).photoId;
        //TestAlbums.printAlbum(albumId, username, user);
        var currentAcl = Acls.getAcl(albumId, username, user.asymKeyPair);
        Acls.removeFromAcl(albumId, username, user.asymKeyPair, user.masterKey, username3, currentAcl.hash);
        var photo3 = Album.insertPhoto(albumId, username, user.masterKey, user.asymKeyPair, 'photo3', trustedHash).photoId;
        //TestAlbums.printAlbum(albumId, username, user);
        trustedHash = Album.deletePhoto(albumId, username, user.masterKey, user.asymKeyPair, photo2, trustedHash);
        TestAlbums.printAlbum(albumId, username, user);
        TestAlbums.printAlbum(albumId, username2, user2);

        log(EJSON.stringify(Album.getAlbumList(username, user.masterKey, user.asymKeyPair, username)));
        log(EJSON.stringify(Album.getAlbumList(username, user.masterKey, user.asymKeyPair, username2)));

        Album.deleteAlbum(albumId, username);
        log(EJSON.stringify(Album.getAlbumList(username, user.masterKey, user.asymKeyPair, username)));
    },

    printAlbum: function(albumId, username, user) {
        var photoList = Album.getPhotos(albumId, username, user.masterKey, user.asymKeyPair);
        log('Album id: ' + albumId);
        log('Username: ' + username);
        log(EJSON.stringify(photoList));
        
        var album = Albums.findOne(albumId);
        if (album === null || typeof(album) === 'undefined')
            throw 'No such album.';
        log(EJSON.stringify(album.photos));
        TestMerkle.printTree(album.merkletreeId);
    }
};
