localStorage = {
};

TestUsers = {
  removeAllUsers: function () {
    UserProfiles.remove({});
    Meteor.users.remove({});
    TestUsers.friendKeys = {};
  },

  friendKeys: {},

  run: function () {
    // remove all users and create some dummy ones
    TestUsers.removeAllUsers();

    var alin = TestUsers.createUser('alinush');
    var prashant = TestUsers.createUser('prashant');
    var sachin = TestUsers.createUser('sachin');
    var lonely = TestUsers.createUser('lonely');
    var inexistent = TestUsers.createUser('inexistent');

    TestUsers.lonelyAdd(lonely, alin);
    TestUsers.lonelyAdd(lonely, prashant);
    TestUsers.lonelyAdd(lonely, sachin);
    
    // NOTE: Race condition here, because of asynchronous method call when adding friend request
    //assert(Friends.isFriendRequestSent(lonely.username, alin.username));
    //assert(Friends.isFriendRequestSent(lonely.username, prashant.username));
    //assert(Friends.isFriendRequestSent(lonely.username, sachin.username));
    //assert(Friends.isFriendRequestSent(lonely.username, inexistent.username) == false);

    TestUsers.createFriendship(alin, prashant);
    TestUsers.createFriendship(alin, sachin);

    TestUsers.createFriendship(sachin, prashant);

    return [ alin, prashant, sachin, lonely ];
  },

  createUser: function (username) {
    var email = username + '@mit.edu';
    var password = username;

    var creds = Utils.registerAccount(username, email, password);

    Trustore.Keys.set('master', creds.masterKey, username);
    Trustore.Keys.setAsym(creds.asymKeyPair, username);

    var profileId = Utils.getProfileId(username);
    var acl = Acls.getAcl(profileId, username, creds.asymKeyPair, 0);
    var aclVer = acl.acl.aclVer;
    var profileKeys = Acls.getContentKeys(profileId, username, creds.asymKeyPair, username, creds.masterKey, acl.hash);
    //console.log('profileKey: ' + stri(profileKey));

    var profile = {
        text: CryptoUtils.encrypt(profileKeys[aclVer-1], 'hello, this is my <b>' + username + '</b> profile')
    };

    var timeStamp = new Date();

    var msg = EJSON.stringify({username: username, aclVer: aclVer, profile: profile, timeStamp: timeStamp});
    var signature = CryptoUtils.signString(creds.asymKeyPair, msg);
        
    UserProfiles.update(profileId, {$set: {aclVer: aclVer, profile: profile, timeStamp: timeStamp, signature: signature}});
    //UserProfiles.insert({ username: username, text: CryptoUtils.encrypt(profileKey, 'hello, this is my <b>' + username + '</b> profile') });

    return creds;
  },

  createFriendship: function (cred1, cred2) {
    console.log('[TestUsers] ' + cred1.username + ' is adding ' + cred2.username);
    var f1 = Friends.addFriend(cred1.username, cred1.asymKeyPair, cred2.username, cred2.asymKeyPair);

    console.log('[TestUsers] ' + cred2.username + ' is accepting ' + cred1.username);
    var f2 = Friends.acceptFriend(cred2.username, cred2.asymKeyPair, cred1.username, cred1.asymKeyPair, f1.date, f1.reqSig);

    // make sure friendKeys are the same!
    //console.log('f1.friendKey: ' + f1.friendKey);
    //console.log('f2.friendKey: ' + f2.friendKey);
    //var masterKey = cred1.masterKey;
    //console.log(masterKey + ' vs. ' + CryptoUtils.decryptKey(masterKey, CryptoUtils.encryptKey(masterKey, masterKey)));

    assert.equal(CryptoUtils.serializeSymKey(f1.friendKey), 
      CryptoUtils.serializeSymKey(f2.friendKey));

    assert.equal(CryptoUtils.serializeSymKey(f1.friendKey),
      CryptoUtils.serializeSymKey(
        Friends.getFriendKey(cred1.username, cred1.masterKey, cred2.username)));

    TestUsers.setFriendKey(cred1, cred2, f1.friendKey);
    return f1.friendKey;
  },

  lonelyAdd: function(lonely, cred) {
    console.log('[TestUsers] ' + lonely.username + ' is adding ' + cred.username);
    Friends.addFriend(lonely.username, lonely.asymKeyPair, cred.username, cred.asymKeyPair);
  },

  setFriendKey: function(user1, user2, frk) {
    TestUsers.friendKeys[user1.username + '|' + user2.username] = frk;
    TestUsers.friendKeys[user2.username + '|' + user1.username] = frk;
  },

  getFriendKey: function(user1, user2) {
    return TestUsers.friendKeys[user1.username + '|' + user2.username];
  },
}
